
// Application CommandLoop

package sptossg ;

import java.util.*;

public class CommandLoop {

    // MAIN - a routine for testing CompanyInfo storage and retrieval
    public static void main (String args[]) {
	new CommandLoop();
    }
    
    // I/O routines for the command loop
    String prompt(String pstr) {
	return prompt(pstr, null);
    }

    // I/O routines for the command loop
    String prompt(String pstr, String possval) {
	if (possval != null) {
	    possval = possval.trim();
	    if (! possval.equals("")) { return possval; }
	}

	byte linebytes[] = new byte[1024];
	System.out.print(pstr + ": ");
	System.out.flush();
	try {
	    System.in.read(linebytes);
	} catch (Exception exc) {
	    System.out.println("exception on read ");

	    return null;
	}

	String line = new String(linebytes);
	return line.trim();
    }

    // separarate words by whitespace
    int wordsep(String line) {
	int last = line.length();

	line = line + " \t";
	last = Math.min(last,line.indexOf(' '));
	last = Math.min(last,line.indexOf('\t'));
	return last;
    }

    // cmdloop - A simple command processor for testing
    public CommandLoop() {
	CompanyInfo curcompany = null;
	Hashtable allcompanies = new Hashtable();

	System.out.println("\nSSG Program Command Line processor");
	System.out.println("Type 'help' for a list of commands\n");

	while (true) {
	    String line = prompt("Command");

	    int sep = wordsep(line);

	    String cmd = line.substring(0,sep).toLowerCase();
	    String params = line.substring(sep).trim();

	    // Exit the command loop
	    if (cmd.equals("quit")) { return; }

	    // list commands
	    if (cmd.equals("help")) {
		System.out.println("Available commands: ");
		System.out.println("def[ine] - Define a DynamicDataSet");
		System.out.println("parse - Parse an HTML file");
		System.out.println("load - load a .SSG file");
		System.out.println("list - List available companies");
		System.out.println("rename - Rename a company");
		System.out.println("use - Switch the current company");
		System.out.println("retr[ieve] - retrieve an attribute");
		System.out.println("attr - list the attributes");
		System.out.println("dump - Display one or all attributes");
		System.out.println("quit");
	    }

	    // Define a DynamicDataSet
	    if (cmd.equals("def") || cmd.equals("define")) {

		sep = wordsep(params);

		String attr = prompt("Attribute Name", params.substring(0,sep));
		String expr = prompt("Attribute Expression", params.substring(sep));

		DynamicDataSet dds = DynamicDataSet.define(attr, expr);
		if (dds == null) {
		    System.out.println("Parse error defining " + attr);
		}
	    }

	    // Initialize some formulas
	    if (cmd.equals("setup")) {

		String attrs[][] =
		{ {"PTP Margin", "100 * [PreTax Inc] / Sales"},
		  {"Sales Increase", "100 * Sales / prev(Sales)"}
		};

		for (int i = 0; i < attrs.length; i++) {
		    String aname = attrs[i][0];
		    String aformula = attrs[i][1];

		    DynamicDataSet.define(aname, aformula);
		    System.out.println("Defined dynamic attribute: " +
				       aname + " => " + aformula);
		}
	    }

	    // toggle debugging
	    if (cmd.equals("debug")) {
		Debug.toggle();
	    }

	    // Parse an HTML file
	    if (cmd.equals("parse")) {
		String coname = prompt("Company", params);

		curcompany = new CompanyInfo();
		allcompanies.put(coname, curcompany);

		try {
		    SPReportParser parser = new SPReportParser(coname + ".html");
		    if (!parser.parse(curcompany)) {
			System.err.println("parse() failed with the message: "
					   + parser.parseError());
		    } else {
			System.out.println(coname + ".html parsed. ");
		    }
		} catch (Exception exc) {
		    System.out.println("Error: " + exc);
		}
	    }

	    // load a .SSG file
	    if (cmd.equals("load")) {
		String coname = prompt("Company", params);

		curcompany = new CompanyInfo();
		allcompanies.put(coname, curcompany);

		curcompany.loadSSGfile(coname + ".ssg");
		System.out.println(coname + ".ssg loaded. ");
	    }

	    // List available companies
	    if (cmd.equals("list")) {
		Enumeration e;

		int i = 0;
		e = allcompanies.keys();
		while (e.hasMoreElements()) {
		    String key = (String) e.nextElement();
		    System.out.println(i++ + ": " + key);
		}
		if (i == 0) {
		    System.out.println("No companies loaded.");
		}
	    }

	    // Rename a company
	    if (cmd.equals("rename")) {
		sep = wordsep(params);

		String coname = prompt("Company", params.substring(0,sep));
		String newname = prompt("New company id", params.substring(sep));

		CompanyInfo co = (CompanyInfo) allcompanies.get(coname);
		if (co == null) {
		    System.out.println("No company named '" + coname + "'");
		} else {		    
		    allcompanies.put(newname, co);
		    allcompanies.remove(coname);
		}
	    }

	    // Switch the current company
	    if (cmd.equals("use")) {
		String coname = prompt("Company", params);

		curcompany = (CompanyInfo) allcompanies.get(coname);
		if (curcompany == null) {
		    System.out.println("No company named '" + coname + "'");
		}
	    }

	    // retrieve an attribute from the current company
	    if (cmd.equals("retr") || cmd.equals("retrieve")) {

		if (curcompany == null) {
		    System.out.println("No current company.");
		} else {

		    String attr = prompt("Attribute", params);
		    String val;
		    String perref;

		    String year = prompt("Year");
		    if (year.equals("")) {
			val = curcompany.get(attr);
			perref = "";
		    } else {
			String period = prompt("Period");
			if (period.equals("")) {
			    val = curcompany.get(attr,year);
			    perref = "(" + year + ")";
			} else {
			    val = curcompany.get(attr,year, period);
			    perref = "(" + year + "," + period + ")";
			}
		    }
		    System.out.println(attr + perref + " = " + val);
		}
	    }

	    // list the attributes of the current company
	    if (cmd.equals("attr")) {
		if (curcompany == null) {
		    System.out.println("No current company.");
		} else {
		    curcompany.dumpAttrs();
		}
	    }

	    // Display the current company
	    if (cmd.equals("dump")) {
		if (curcompany == null) {
		    System.out.println("No current company.");
		} else {
		    String attr = prompt("Attribute", params);
		    if (attr.equals("")) {
			curcompany.dump();
		    } else {
			curcompany.dump(attr);
		    }
		}
	    }
	}
    }

}
